/*  FreeJ
 *  (c) Copyright 2001 Denis Roio aka jaromil <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <layer.h>
#include <lubrify.h>

void Layer::_init() {
  _w = w; _h = h;
  _pitch = pitch;
  _size = size;
}

bool Layer::add_filter(Filter *newfilt) {
  char temp[256];
  bool res = false;
  func("Layer::add_filter()");

  if(newfilt==NULL) {
    error("invalid NULL filter");
    return(res);
  }

  if(!newfilt->bpp_ok(bpp)) {
    sprintf(temp,"%s filter does'nt support %ubpp",newfilt->name,bpp);
    warning(temp);
    return(res);
  }

  /* here pointers are used to let filters change the size */
  newfilt->w = &w; newfilt->h = &h;
  newfilt->pitch = &pitch; newfilt->size = &size;

  newfilt->bpp = bpp;

  newfilt->fps = fps; /* this is a pointer from the screen */

  snprintf(temp,256,
	   "%s filter registered :: w[%u] h[%u] bpp[%u] size[%u]"
	   ,newfilt->name,w,h,bpp,size);
  notice(temp);

  if(!newfilt->initialized) {
    res = newfilt->init();
    newfilt->initialized = true;
  } else res = true;

  /* add the filter to the linklist */
  filters.add(newfilt);

  return(res);

}

void Layer::clear_filters() {
  filters.clear();
}

bool Layer::cafudda() {

  void *res = get_buffer();

  /* restore original size info so that it
     can be changed from the filters */
  w = _w; h = _h; pitch = _pitch; size = _size;

  Filter *filt = (Filter *)filters.begin();

  while(filt!=NULL) {
    res = filt->process(res);
    filt = (Filter *)filt->next;
  }

  if(pitch==screen->pitch)
    mmxcopy(res,screen->get_surface(),size);
  else
    mmxblit(res,screen->coords(x,y),h,pitch,screen->pitch); 
  /* pitch is width in bytes */
  
  return(true);
}
